﻿using GalaSoft.MvvmLight;
using System.Windows.Input;
using System.Collections.ObjectModel;
using GalaSoft.MvvmLight.Command;
using System;
using System.DirectoryServices;
using ActiveDs;
using System.DirectoryServices.ActiveDirectory;

namespace AD_Explorer.ViewModel
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class MainViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the MainViewModel class.
        /// </summary>
        public MainViewModel()
        {
            this.ConnectCommand = new RelayCommand<string>((ServerText) => this.Connect(ServerText), (ServerText)=>true);
            
        }

        #region Properties
        #region ServerString
        /// <summary>
        /// The <see cref="ServerString" /> property's name.
        /// </summary>
        public const string ServerStringPropertyName = "ServerString";

        private string _serverStringProperty = string.Empty;

        /// <summary>
        /// Gets the ServerString property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public string ServerString
        {
            get
            {
                return _serverStringProperty;
            }

            set
            {
                if (_serverStringProperty == value)
                {
                    return;
                }
                _serverStringProperty = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(ServerStringPropertyName);
            }
        }
        #endregion
        
        #region SelectedDirectoryEntry
        /// <summary>
        /// The <see cref="SelectedDirectoryEntry" /> property's name.
        /// </summary>
        public const string SelectedDirectoryEntryPropertyName = "SelectedDirectoryEntry";

        private DirectoryEntryViewModel _selectedDirectoryEntryProperty = null;

        /// <summary>
        /// Gets the SelectedDirectoryEntry property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public DirectoryEntryViewModel SelectedDirectoryEntry
        {
            get
            {
                return _selectedDirectoryEntryProperty;
            }
            set
            {
                if (_selectedDirectoryEntryProperty == value)
                {
                    return;
                }
                _selectedDirectoryEntryProperty = value;
                // Update bindings, no broadcast
                RaisePropertyChanged(SelectedDirectoryEntryPropertyName);
            }
        }
        #endregion

        #region ADRoot
        /// <summary>
        /// The <see cref="ADRoot" /> property's name.
        /// </summary>
        public const string ADRootPropertyName = "ADRoot";

        private ObservableCollection<DirectoryEntryViewModel> _ADRootProperty = new ObservableCollection<DirectoryEntryViewModel>();

        /// <summary>
        /// Gets the ADRoot property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public ObservableCollection<DirectoryEntryViewModel> ADRoot
        {
            get
            {
                return _ADRootProperty;
            }

            set
            {
                if (_ADRootProperty == value)
                {
                    return;
                }
                _ADRootProperty = value;
                // Update bindings, no broadcast
                RaisePropertyChanged(ADRootPropertyName);
            }
        }
        #endregion

        #region AdvancedView
        /// <summary>
        /// The <see cref="AdvancedViewActivated" /> property's name.
        /// </summary>
        public const string AdvancedViewActivatedPropertyName = "AdvancedViewActivated";

        private bool _advancedViewActivated = false;

        /// <summary>
        /// Gets the AdvancedViewActivated property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public bool AdvancedViewActivated
        {
            get
            {
                return _advancedViewActivated;
            }

            set
            {
                if (_advancedViewActivated == value)
                {
                    return;
                }
                _advancedViewActivated = value;
                
                // Update bindings, no broadcast
                RaisePropertyChanged(AdvancedViewActivatedPropertyName);
            }
        }
        #endregion
        #endregion

        #region Commands
        #region ConnectCommand
        /// <summary>
        /// The <see cref="ConnectCommand" /> property's name.
        /// </summary>
        public const string ConnectCommandPropertyName = "ConnectCommand";

        private ICommand _connectCommand = null;

        /// <summary>
        /// Gets the ConnectCommand property.
        /// Connects to a given AD Server object, and puts the object as a root
        /// </summary>
        public ICommand ConnectCommand
        {
            get
            {
                return _connectCommand;
            }

            set
            {
                if (_connectCommand == value)
                {
                    return;
                }
                _connectCommand = value;
                RaisePropertyChanged(ConnectCommandPropertyName);
            }
        }
        #endregion
        #endregion

        #region Methods
        private void Connect(string ServerText)
        {
            DirectoryEntry entry = new DirectoryEntry("LDAP://" + ServerString);
            ADRoot.Add(new DirectoryEntryViewModel(entry));
        }
        #endregion

        public override void Cleanup() 
        {
            // Clean own resources if needed
            base.Cleanup();
        }
    }
}